import numpy as np
import os
from subprocess import Popen, PIPE
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import scipy.constants as sc
from scipy.optimize import curve_fit

globalPointMarker = 'o'
globalMarkerSize = 5
globalTextSize = 20

def calcTcolor(T, Tmin, Tmax):
    """
    Calculates the color that the corresponding line should be plotted with based on the
    idea that the coldest temperature should be completely blue RGB=(0,0,255) and the warmest
    temperature should be completely red RGB=(255,0,0)
    
    Input
    T: temperature of the curve that is being plotted
    D: dictionary containing all temperatures

    Output
    RGB: a tuple containing (R,G,B)-values for the color to be used
    """
    
    T18 = Tmin + 1/8*(Tmax-Tmin)
    T38 = Tmin + 3/8*(Tmax-Tmin)
    T58 = Tmin + 5/8*(Tmax-Tmin)
    T78 = Tmin + 7/8*(Tmax-Tmin)
    
    R, B, G = 0, 0, 0
    if T >= T78:
        p = (T-T78)/(Tmax-T78)
        R = 1-0.5*p
    elif T >= T58:
        p = (T-T58)/(T78-T58)
        R = 1
        G = 1-p
    elif T >= T38:
        p = (T-T38)/(T58-T38)
        R = p
        G = 1
        B = 1-p
    elif T >= T18:
        p = (T-T18)/(T38-T18)
        G = p
        B = 1
    else:
        p = (T-Tmin)/(T18-Tmin)
        B = 0.5+0.5*p
    
    return (R,G,B)
    
def numOfFreqRead(Temps):
    """
    Calculates the number of frequencies measured for each temperature in a PPMS data file
    
    Input
    Temps: a temperature column from a PPMS data file
    
    Output
    numOfFreq: an integer number of frequencies that has been measured
    """

    Temps = np.round(Temps, 1)*10
    Temps = np.array([int(x) for x in list(Temps)])
    bins = np.bincount(Temps)
    numOfFreq = int(np.max(bins))
    
    return numOfFreq

def readACDATAtoARRAY(fileName, dataOrigin='PPMS'):
    """Reads the output file from a PPMS ACMS measurement at AU and gives the result as a dictionary
    Input
    fileName: the name of the file to be read from
    numberofFreq: the number of frequency points for each temperature
    
    Output
    resultDict: a Python dictionary containing the specified below
    """
    
    colsToUse, rowsToSkip, delimiterToUse = 0, 0, 0
    if dataOrigin == 'PPMS':
        colsToUse = (2,3,4,5,8,9)
        rowsToSkip = 21
        delimiterToUse = ','
    
    f = open(fileName, 'r')
    names = f.readlines()
    f.close()
    
    names = [names[rowsToSkip-1].split(delimiterToUse)[n] for n in colsToUse]
    
    # Load the measured data into an array. Loading columns 'Temperature (K)', 'Magnetic Field (Oe)', 'Frequency (Hz)', 'Amplitude (Oe)', 'M' (emu)', 'M'' (emu)'
    D = np.loadtxt(fileName, delimiter=delimiterToUse, skiprows=rowsToSkip, usecols=colsToUse)
    
    # Reading the number of frequencies that have been measured
    numberofFreq = numOfFreqRead(D[:,0])
    #print('Splitting data according to {} frequencies per temperature'.format(numberofFreq))
    
    # Calculating how many different temperatures have been measured
    splitIntoArrays = int(len(D[:,0])/numberofFreq)
    
    # Splitting the original array into a list of arrays, each with its own temperature
    D = np.split(D, splitIntoArrays, axis=0)

    # Making a dictionary containing all array data. Each entry in the dictionary is a dictionary in itself,
    # which contains the data with the same temperature.
    resultDict = {}
    for d in D:
        T = np.mean(d[:,0]).round(1)
        temperatureDict = {}
        temperatureDict['T (K)'] = T
        for n in range(len(names)):
            temperatureDict[names[n]]=d[:,n]
        resultDict['{}K'.format(T)] = temperatureDict
        
    return resultDict

def calculateX(dict, molWeight, sampleMass, filmMass, setup='capsule'):
    """Adds lists of X' and X'' to the dictionary read from the AC data
    Input
    dict: the dictionary output from function readACDATAtoARRAY
    molWeight: molar weight of the sample in g/mol
    sampleMass: mass of the sample used in g
    filmMass: mass of the film used to wrap the sample in mg
    
    Output
    dict: the modified dictionary
    """
    
    Xd_capsule, Xd_film = 0, 0
    
    if setup == 'capsule':
        Xd_capsule = -1.8*10**-8 # unit: emu/Oe
        Xd_film = 6.47*10**-10 # unit: emu/(Oe*mg)
        
    else:
        print('Setup "{}" not recognized. Return.'.format(setup))
        return None
    
    Xd_sample = -0.6*10**-6
    
    # Modifying the individual dictionaries one at a time
    for key in dict.keys():
        # Read the content of the current dictionary into variable d
        d = dict[key]
        H = d['Amplitude (Oe)']
        H0 = d['Magnetic Field (Oe)']
        Mp = d["M' (emu)"]
        Mpp = d["M'' (emu)"]
        
        # Calculate X' and X'' from M' and M''
        Xp = (Mp - Xd_capsule*H - Xd_film*filmMass*H)*molWeight/(sampleMass*H) - Xd_sample*molWeight
        Xpp = Mpp/(sampleMass*H)*molWeight
        
        d["X' (emu/(Oe*mol))"] = np.array(Xp)
        d["X'' (emu/(Oe*mol))"] = np.array(Xpp)
    
    return dict    
    
def CCFITStartingGuesses(D):
    """
    Reads the dictionary prepared by readACDATAtoARRAY and estimates starting parameters for CCFit based on the CCFit manual
    
    Input
    D: dictionary containing PPMS data in the form as returned by readACDATAtoARRAY
    
    Output:
    res: tuple containing (Xs, Xt, tau, alpha)
    """
    
    # Reading the lowest measured temperature and getting the result as a string
    T = sorted([float(x[0:-1]) for x in list(D.keys())])[0] # the lowest temperature
    Ts = '{}K'.format(T) # the lowest temperature as a string
    
    # Extracting the data from the lowest temperature measurement
    v = D[Ts]['Frequency (Hz)']
    Xp = D[Ts]["X' (emu/(Oe*mol))"]
    Xpp = D[Ts]["X'' (emu/(Oe*mol))"]
    iXppMax = np.argmax(np.array(Xpp))
    tau = 1/(2*np.pi*v[iXppMax])
    Xs = 0
    Xt = Xp[0]
    alpha = 0.1
    
    res = (Xs, Xt, tau, alpha)

    return res

def prepareCCFITinput(D, processes=1):
    """Writes an input file for CCfit from the Chilton group"""
    
     # Sorting the temperatures for printing of the cc-fit input file
    Ts = list(D.keys())
    Ts = [float(t[0:-1]) for t in Ts]
    Ts.sort()
    
    # Opening input file
    f = open('ccinput.dat', 'w')
    
    # Reading the number of temperatures and frequencies from the data and writes them to file head
    temperatures = len(Ts)
    frequencies = len(D[list(D.keys())[0]]['Frequency (Hz)'])
    f.write('{} {} {}\n'.format(processes, temperatures, frequencies))
    
    # Inputting starting parameters into file head
    f.write('{} {} {} {}\n'.format(*CCFITStartingGuesses(D)))
    
    # Writing the frequency and susceptibility data to the file according to the manual specifications
    for n in range(len(D['{}K'.format(Ts[0])]['Frequency (Hz)'])):
        s = '{} '.format(D['{}K'.format(Ts[0])]['Frequency (Hz)'][n])
        for T in Ts:
            Xp = D['{}K'.format(T)]["X' (emu/(Oe*mol))"][n]
            Xpp = D['{}K'.format(T)]["X'' (emu/(Oe*mol))"][n]
            s += '{} {} '.format(Xp, Xpp)
        s += '\n'
        f.write(s)
    
    f.close()
    
def runCCFIT(D):
    """
    Runs the program CCFit in the folder of the script
    """
    
    if 'ccinput.dat' not in os.listdir():
        prepareCCFITinput(D)
        print('Input file for cc-fit not found. Preparing file using 1 relaxation process.')
    
    CCFIT = Popen('cc-fit ccinput.dat', stdin=PIPE, stdout=PIPE, stderr=PIPE)
    CCFIT.stdin.write(b'\n')
    out, err = CCFIT.communicate()
    
    sortedTemps = ['{}K'.format(T) for T in sorted([float(x[0:-1]) for x in list(D.keys())])]
    data = readCCout()
    
    n = 0
    while n < len(sortedTemps):
        D[sortedTemps[n]]['ccXs'] = data[n,0]
        D[sortedTemps[n]]['ccXt'] = data[n,1]
        D[sortedTemps[n]]['cctau'] = data[n,2]
        D[sortedTemps[n]]['ccalpha'] = data[n,3]
        D[sortedTemps[n]]['ccresidual'] = data[n,4]
        n += 1
    
    return D
    
def readCCout():
    """
    Reads the .out-file containing the fitted parameters from CCFit
    
    Output
    data: a n-by-5 array containing the fitted parameters and residuals from the .out file
    """

    inputName = 'ccinput.dat'
    ccextension = '_cc-fit.out'
    
    data = np.loadtxt(inputName+ccextension, skiprows = 13)
    
    return data
    
def Xp_(v, Xs, Xt, tau, alpha):
    """
    Calculates the function X' [chi prime] as specified in Molecular Nanomagnets eq. 3.27
    
    Input:
    v: frequency of the AC field
    Xs: adiabatic limit of susceptibility
    Xt: isothermal limit of susceptibility
    tau: relaxation time of the system
    alpha: width of relaxation time distribution
    
    Output
    Xp: the function value at the specified frequency
    """

    w = 2*np.pi*v
    
    Xp = Xs + (Xt - Xs)*(1 + (w*tau)**(1-alpha)*np.sin(np.pi*alpha/2))/(1 + 2*(w*tau)**(1-alpha)*np.sin(np.pi*alpha/2) + (w*tau)**(2-2*alpha))
    
    return Xp

def Xp_dataset(params, v):
    tau = params['tau']
    alpha = params['alpha']
    Xt = params['Xt']
    Xs = params['Xs']
    return Xp_(v, Xs, Xt, tau, alpha)
    
def Xpp_dataset(params, v):
    tau = params['tau']
    alpha = params['alpha']
    Xt = params['Xt']
    Xs = params['Xs']
    return Xpp_(v, Xs, Xt, tau, alpha)

def Xpp_(v, Xs, Xt, tau, alpha):
    """
    Calculates the function X'' [chi double-prime] as specified in Molecular Nanomagnets eq. 3.27
    
    Input:
    v: frequency of the AC field
    Xs: adiabatic limit of susceptibility
    Xt: isothermal limit of susceptibility
    tau: relaxation time of the system
    alpha: width of relaxation time distribution
    
    Output
    Xpp: the function value at the specified frequency
    """

    w = 2*np.pi*v
    
    Xpp = (Xt - Xs)*(w*tau)**(1-alpha)*np.cos(np.pi*alpha/2)/(1 + 2*(w*tau)**(1-alpha)*np.sin(np.pi*alpha/2) + (w*tau)**(2-2*alpha))

    return Xpp

def Xp_dataset(params, v):
    """
    Wrapper for Xp_ to use with lmfit.
    
    Input:
    params: Instance of lmfit.Parameters containing elements Xs, Xt, alpha and tau as defined under function "Xp_"
    
    Output:
    See output of function "Xp_"
    """
    
    tau = params['tau']
    alpha = params['alpha']
    Xt = params['Xt']
    Xs = params['Xs']
    return Xp_(v, Xs, Xt, tau, alpha)
    
def Xpp_dataset(params, v):
    """
    Wrapper for Xpp_ to use with lmfit.
    
    Input:
    params: Instance of lmfit.Parameters containing elements Xs, Xt, alpha and tau as defined under function "Xpp_"
    
    Output:
    See output of function "Xpp_"
    """
    
    tau = params['tau']
    alpha = params['alpha']
    Xt = params['Xt']
    Xs = params['Xs']
    return Xpp_(v, Xs, Xt, tau, alpha)

def plotXppvsFreq(D, addFit=False):
    """Plots frequency vs. X'' for each temperature"""
    
    Ts = sorted([float(x[0:-1]) for x in list(D.keys())])
    Tmax = Ts[-1]
    Tmin = Ts[0]
    
    fig = plt.figure()
    ax = fig.add_subplot(111)
    
    for key in D.keys():
        d = D[key]
        if addFit:
            d = D[key]
            Xs = d['ccXs']
            Xt = d['ccXt']
            tau = d['cctau']
            alpha = d['ccalpha']
            v = np.linspace(d['Frequency (Hz)'][0], d['Frequency (Hz)'][-1], 1000)
            ax.semilogx(v, Xpp_(v, Xs, Xt, tau, alpha), c=calcTcolor(d['T (K)'], Tmin, Tmax))
            ax.semilogx(d['Frequency (Hz)'], d["X'' (emu/(Oe*mol))"], marker=globalPointMarker,
                                                                      markerfacecolor='None',
                                                                      markersize=globalMarkerSize,
                                                                      linestyle='None',
                                                                      markeredgecolor='k',
                                                                      linewidth=1)
        
        else:
            ax.semilogx(d['Frequency (Hz)'], d["X'' (emu/(Oe*mol))"], c=calcTcolor(d['T (K)'], Tmin, Tmax), marker='o', linestyle='None')
    
    large_T = mpatches.Patch(color=(0.5,0,0), label='{}K'.format(Tmax))
    small_T = mpatches.Patch(color=(0,0,0.5), label='{}K'.format(Tmin))

    ax.tick_params(labelsize=globalTextSize)    
    ax.set_xlabel(r"$\nu$ [$Hz$]", fontsize=globalTextSize)
    ax.set_ylabel(r"$\chi$'' [$\frac{emu}{Oe  mol}$]", fontsize=globalTextSize)
    ax.legend(handles=[small_T, large_T], fontsize=globalTextSize)
    
    return fig
    
def ColeColePlot(D, addFit=False):
    """Plots a Cole-Cole plot of the measured data"""

    Ts = sorted([float(x[0:-1]) for x in list(D.keys())])
    Tmax = Ts[-1]
    Tmin = Ts[0]
    
    fig = plt.figure()
    ax = fig.add_subplot(111)
    
    for key in D.keys():
        d = D[key]
        if addFit:
            d = D[key]
            Xs = d['ccXs']
            Xt = d['ccXt']
            tau = d['cctau']
            alpha = d['ccalpha']
            v = np.linspace(d['Frequency (Hz)'][0], d['Frequency (Hz)'][-1], 1000)
            ax.plot(Xp_(v, Xs, Xt, tau, alpha), Xpp_(v, Xs, Xt, tau, alpha), c=calcTcolor(d['T (K)'], Tmin, Tmax), linestyle='-')
            ax.plot(d["X' (emu/(Oe*mol))"], d["X'' (emu/(Oe*mol))"], marker=globalPointMarker,
                                                                     markerfacecolor='None',
                                                                     markersize=globalMarkerSize,
                                                                     linestyle='None',
                                                                     markeredgecolor='k',
                                                                     linewidth=1)
    
        else:
            ax.plot(d["X' (emu/(Oe*mol))"], d["X'' (emu/(Oe*mol))"], c=calcTcolor(d['T (K)'], Tmin, Tmax), marker='o', linestyle='None')

    large_T = mpatches.Patch(color=(0.5,0,0), label='{}K'.format(Tmax))
    small_T = mpatches.Patch(color=(0,0,0.5), label='{}K'.format(Tmin))
    
    ax.tick_params(labelsize=globalTextSize)
    ax.set_xlabel(r"$\chi$' [$\frac{emu}{Oe  mol}$]", fontsize=globalTextSize)
    ax.set_ylabel(r"$\chi$'' [$\frac{emu}{Oe  mol}$]", fontsize=globalTextSize)
    ax.legend(handles=[small_T, large_T], fontsize=globalTextSize)
        
    return fig

def tauvsTempPlot(D):
    """
    Plots the relaxation time vs. temperature for the measured data.
    
    Input
    D: the dictionary that has been modified by CCfit to incorporate tau
    
    Output:
    res: a tuple containing the arrays (T, tau)
    """
    
    T = sorted([float(x[0:-1]) for x in list(D.keys())])
    tau = []
    
    for t in T:
        tau.append(D['{}K'.format(t)]['cctau'])
    
    T, tau = np.array(T), np.array(tau)
    
    plt.plot(1/T, np.log(tau), 'bo')
    plt.show()
    
    return tuple([T, tau])

def readXppvsT(D):

    Ts = []
    for key in D.keys():
        Ts.append(float(key[:-1]))
    Ts = sorted(Ts)
    
    Q = {}
    W = D['{}K'.format(Ts[0])]['Frequency (Hz)']
    for w in W:
        Q['{}Hz'.format(round(w,2))] = {'Temp (K)':[], 'Xp':[], 'Xpp':[]}
    for T in Ts:
        w = D['{}K'.format(T)]['Frequency (Hz)']
        Xp = D['{}K'.format(T)]["X' (emu/(Oe*mol))"]
        Xpp = D['{}K'.format(T)]["X'' (emu/(Oe*mol))"]
        for n in range(len(w)):
            Q['{}Hz'.format(round(w[n],2))]['Temp (K)'].append(T)
            Q['{}Hz'.format(round(w[n],2))]['Xp'].append(Xp[n])
            Q['{}Hz'.format(round(w[n],2))]['Xpp'].append(Xpp[n])
    
    return Q

def makeOrbachfit(fileName, minT, maxT, U_guess, t0_guess, U_unit='K'):
    """
    Reads the (T,tau)-data from <fileName> and does a fitting to an
    Orbach-process.
    
    Input
    fileName: name of the file in the format printed by makeTvsTauFile
    minT: minimum temperature used for the fit
    maxT: maximum temperature used for the fit
    t0_guess: guess for the t0-parameter in the Orbach model
    U_guess: guess for the U-parameter in the Orbach-model
    U_guess_unit: unit of the guess given for U-parameter (default: 'K' (Kelvin))
    
    Output
    P: parameters and their uncertainties from the modeling
    """
    
    kB = sc.Boltzmann
    
    D = np.loadtxt(fileName, skiprows=1)
    T = D[:,0]
    tau = D[:,1]
    
    # Calculating the values to use for the fit
    T_used = np.array([x for x in T if x>=minT and x<=maxT])
    T_not_used = np.array([x for x in T if x<minT or x>maxT])
    
    # Calculating the values not to use for the fit
    tau_used = np.array([tau[np.where(T==x)[0][0]] for x in T_used])
    tau_not_used = np.array([tau[np.where(T==x)[0][0]] for x in T_not_used])
    
    # Recalculating U_guess
    if U_unit=='K':
        U_guess = U_guess*kB
    elif U_unit=='J':
        pass
    else:
        print('Wrong unit given. Ending.')
        return
    
    # Fitting
    A = curve_fit(OrbachTau, T_used, tau_used, [U_guess, t0_guess])
    
    U_fitted = A[0][0]
    t0_fitted = A[0][1]
    
    uncertainties = np.sqrt(np.diag(A[1]))
    U_sigma = uncertainties[0]
    t0_sigma = uncertainties[1]
    
    # Recalculating U_fitted
    if U_unit=='K':
        U_fitted = U_fitted/kB
        U_sigma = U_sigma/kB
    elif U_unit=='J':
        pass
    
    plt.title('no_linear')
    plt.plot(1/T_used, np.log(OrbachTau(T_used, A[0][0], A[0][1])))
    plt.plot(1/T_used, np.log(tau_used),'k*')
    plt.plot(1/T_not_used, np.log(tau_not_used), 'r*')
    
    P = (U_fitted, U_sigma, t0_fitted, t0_sigma)
    print('no_linear')
    print('Ueff = {:3.2f} +/- {:3.2f}\nt0 = {:3.2f} +/- {:3.2f}'.format(P[0], P[1], P[2], P[3]))
    plt.show()
    
    return P

def OrbachTau(T, Ueff, t0):
    """
    Returns the Orbach relaxation time at a given temperature according to the parameters Ueff and t0
    
    Input
    T: temperature list or float
    Ueff: effective energy barrier for Orbach relaxation in joules
    t0: the maximal Orbach relaxation rate
    
    Output
    tau: a list or float giving the Orbach relaxation times
    """
    kB = sc.Boltzmann
    
    tau = t0*np.exp(Ueff/(kB*T))
    
    return tau

def RamanTau(T, Ct, n):
    """
    Returns the Raman relaxation time at a given temperature according to the parameters Ct and n
    
    Input
    T: temperature list or float
    Ct: characteristic constant for Raman relaxation
    n: the Raman exponent of the relaxation
    
    Output
    tau: a list or float giving the Raman relaxation times
    """

    tau = Ct*T**(-n)
    
    return tau
    
def QTTau(T, tQT):
    """
    Returns the quantum tunneling relaxation time at a given temperature according to the parameter tQT
    
    Input
    T: temperature list or float
    tQT: quantum tunneling relaxation time
    """
    
    tau = tQT
    
    return tau
    
def getParameterGuesses(T, tau):
    """
    Calculates guesses for optimal fitting parameters to begin the fit
    
    Input
    T: temperature array
    tau: relaxation time array
    
    Output
    guess: dictionary of guessed parameters for the relaxation functions
    """
    
    kB = sc.Boltzmann
    
    # Obtaining points for Orbach parameter guessing
    T1, tau1 = T[-1], tau[-1]
    T2, tau2 = T[-2], tau[-2]
    
    # Calculating guesses for Orbach parameters
    Ueff_guess = (np.log(tau2) - np.log(tau1))/(1/T2-1/T1)*kB
    
    t0_guess = tau1*np.exp(-Ueff_guess/(kB*T1))
    
    # Obtaining points for Raman parameter guessing
    l = len(T)
    index1, index2 = 0,0
    if l/2 % 1 == 0:
        index1, index2 = int(l/2), int(l/2+1)
    else:
        index1, index2 = int(l/2-0.5), int(l/2+0.5)
    
    T1, tau1 = T[index1], tau[index1]
    T2, tau2 = T[index2], tau[index2]    
    
    # Calculating guesses for Raman parameters
    n_guess = (np.log(tau1) - np.log(tau2))/(np.log(T2) - np.log(T1))
    
    Cr_guess = tau1*T1**n_guess
    
    # Extracting guess for QT parameter
    tQT_guess = tau[0]
    
    guess = {'Ueff': Ueff_guess,
            't0': t0_guess,
            'n': n_guess,
            'Cr': Cr_guess,
            'tQT': tQT_guess}
    
    return guess
    
def makeTvsTauFile(D, fileName, saveFile=True):
    """
    Prints the data from the dictionary D into a file containing 2 columns of
    temperature data and relaxation time data
    
    Input
    D: dictionary as returned from runCCFIT
    fileName: name to give to the new file that the function creates
    
    Output
    T: sorted array of measured temperatures
    tau: array matching the sorting of T with fitted relaxation times
    """
    
    T = sorted([float(x[:-1]) for x in D.keys()])
    tau = [D['{}K'.format(x)]['cctau'] for x in T]
    
    if saveFile:
        f = open(fileName, 'w')
        f.write('{:>6s}{:>15s}\n'.format('T', 'tau'))
    
        for n in range(len(T)):
            f.write('{:6.3f}{:15.6e}\n'.format(T[n], tau[n]))
        
        f.close()
        
        print('Data written to file {}'.format(fileName))
    
    return np.array(T), np.array(tau)

def _QT(T, tQT):
    """
    Basic function for calculating relaxation time due to
    quantum tunneling
    
    Input
    T: temperature for the calculation
    tQT: characteristic time for quantum tunneling
    
    Output
    tau: relaxation time due to quantum tunneling
    """
    
    tau = tQT
    
    return tau

def _R(T, Cr, n):
    """
    Basic function for calculating relaxation time due to
    the Raman mechanism
    
    Input
    T: temperature for the calculation
    Cr: Raman pre-factor
    n: Raman exponent
    
    Output
    tau: relaxation time due to the Raman mechanism
    """
    
    tau = Cr*T**-n

    return tau
    
def _O(T, t0, Ueff):
    """
    Basic function for calculating relaxation time due to
    the Orbach relaxation mechanism
    
    Input
    T: temperature for the calculation
    t0: characteristic time for quantum tunneling
    Ueff: effective energy barrier to thermal relaxation
    
    Output
    tau: relaxation time due to the Orbach mechanism
    """
    
    kB = sc.Boltzmann
    
    tau = t0*np.exp(Ueff/(kB*T))
    
    return tau
    
def _QT_log(T, tQT):
    """
    Wrapper function to function _QT that computes the logarithmic
    relaxation time due to quantum tunneling.
    
    See help(_QT) for more
    """
    
    return np.log(_QT(T, tQT))

def _R_log(T, Cr, n):
    """
    Wrapper function to function _R that computes the logarithmic
    relaxation time due to the Raman mechanism.
    
    See help(_R) for more
    """    
    
    return np.log(_R(T, Cr, n))
    
def _O_log(T, t0, Ueff):
    """
    Wrapper function to function _O that computes the logarithmic
    relaxation time due to the Orbach mechanism.
    
    See help(_O) for more
    """
    
    return np.log(_O(T, t0, Ueff))
    
def _QTR(T, tQT, Cr, n):
    """
    Wrapper function that computes the combined effect of a quantum
    tunneling mechanism and the Raman mechanism
    
    See help(_QT) and help(_R) for more
    """
    
    w = 1/_QT(T, tQT) + 1/_R(T, Cr, n)
    
    tau = 1/w
    
    return np.log(tau)

def _QTO(T, tQT, t0, Ueff):
    """
    Wrapper function that computes the combined effect of a quantum
    tunneling mechanism and the Orbach mechanism
    
    See help(_QT) and help(_O) for more
    """
    
    w = 1/_QT(T, tQT) + 1/_O(T, t0, Ueff)
    
    tau = 1/w
    
    return np.log(tau)
    
def _RO(T, Cr, n, t0, Ueff):
    """
    Wrapper function that computes the combined effect of a Raman
    mechanism and the Orbach mechanism
    
    See help(_R) and help(_O) for more
    """
    
    w = 1/_R(T, Cr, n) + 1/_O(T, t0, Ueff)
    
    tau = 1/w
    
    return np.log(tau)

def _QTRO(T, tQT, Cr, n, t0, Ueff):
    """
    Wrapper function that computes the combined effect of a quantum
    tunneling mechanism, the Raman mechanism and the Orbach mechanism
    
    See help(_QT), help(_R) and help(_O) for more
    """
    
    w = 1/_QT(T, tQT) + 1/_R(T, Cr, n) + 1/_O(T, t0, Ueff)
    
    tau = 1/w
    
    return np.log(tau)

def getStartParams(guess, fitType='QTRO'):
    
    p0 = 0
    if fitType=='QT':
        p0 = [guess['tQT']]
    elif fitType=='R':
        p0 = [guess['Cr'], guess['n']]
    elif fitType=='O':
        p0 = [guess['t0'], guess['Ueff']]
    elif fitType=='QTR':
        p0 = getStartParams(guess, fitType='QT') + getStartParams(guess, fitType='R')
    elif fitType=='QTO':
        p0 = getStartParams(guess, fitType='QT') + getStartParams(guess, fitType='O')
    elif fitType=='RO':
        p0 = getStartParams(guess, fitType='R') + getStartParams(guess, fitType='O')
    elif fitType=='QTRO':
        p0 = [guess['tQT'], guess['Cr'], guess['n'], guess['t0'], guess['Ueff']]
    else:
        print('fitType parameter did not correspond to any correct one')
        
    return p0

def getFittingFunction(fitType='QTRO'):
    
    f = 0
    if fitType=='QT':
        f = _QT_log
    elif fitType=='R':
        f = _R_log
    elif fitType=='O':
        f = _O_log
    elif fitType=='QTR':
        f = _QTR
    elif fitType=='QTO':
        f = _QTO
    elif fitType=='RO':
        f = _RO
    elif fitType=='QTRO':
        f = _QTRO
    else:
        print('fitType parameter did not correspond to any correct one')
        
    return f
    
def readPopt(popt, pcov, fitType='QTRO'):

    p_fit = 0
    if fitType=='QT':
        p_fit = {'params': popt, 'sigmas': np.sqrt(np.diag(pcov)), 'quantities': ['tQT']}
    elif fitType=='R':
        p_fit = {'params': popt, 'sigmas': np.sqrt(np.diag(pcov)), 'quantities': ['Cr', 'n']}
    elif fitType=='O':
        p_fit = {'params': popt, 'sigmas': np.sqrt(np.diag(pcov)), 'quantities': ['t0', 'Ueff']}
    elif fitType=='QTR':
        p_fit = {'params': popt, 'sigmas': np.sqrt(np.diag(pcov)), 'quantities': ['tQT', 'Cr', 'n']}
    elif fitType=='QTO':
        p_fit = {'params': popt, 'sigmas': np.sqrt(np.diag(pcov)), 'quantities': ['tQT', 't0', 'Ueff']}
    elif fitType=='RO':
        p_fit = {'params': popt, 'sigmas': np.sqrt(np.diag(pcov)), 'quantities': ['Cr', 'n', 't0', 'Ueff']}
    elif fitType=='QTRO':
        p_fit = {'params': popt, 'sigmas': np.sqrt(np.diag(pcov)), 'quantities': ['tQT', 'Cr', 'n', 't0', 'Ueff']}
    
    return p_fit
    
def fitRelaxation(fileName, tempRange, fitType='QTRO'):
    """
    Documentation to come
    """
    
    D = np.loadtxt(fileName, skiprows=1)
    T = D[:,0]
    tau = D[:,1]
    minT = tempRange[0]
    maxT = tempRange[1]
    
    # Calculating the values to use for the fit
    T_used = np.array([x for x in T if x>=minT and x<=maxT])
    T_not_used = np.array([x for x in T if x<minT or x>maxT])
    
    T_used_space = np.linspace(T_used[0], T_used[-1], 1000)
    
    # Calculating the values not to use for the fit
    tau_used = np.array([tau[np.where(T==x)[0][0]] for x in T_used])
    tau_not_used = np.array([tau[np.where(T==x)[0][0]] for x in T_not_used])
    
    # Obtaining automated guesses for fitting parameters
    guess = getParameterGuesses(T, tau)
    f = getFittingFunction(fitType=fitType)
    p0 = getStartParams(guess, fitType=fitType)
    
    # Making a fit to the selected data
    popt, pcov = curve_fit(f, T_used, np.log(tau_used), p0)
    p_fit = readPopt(popt, pcov, fitType=fitType)
    
    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.plot(1/T_used_space, np.ones(T_used_space.shape)*f(T_used_space, *p_fit['params']), 'b-', label='Fitted')
    ax.plot(1/T_used, np.log(tau_used), marker=globalPointMarker,
                                        markersize=globalMarkerSize,
                                        markeredgecolor='k',
                                        markerfacecolor='None',
                                        linestyle='None')
                                        
    ax.plot(1/T_not_used, np.log(tau_not_used), marker=globalPointMarker,
                                                markersize=globalMarkerSize,
                                                markeredgecolor='r',
                                                markerfacecolor='None',
                                                linestyle='None')
    
    ax.set_xlabel(r'$\frac{1}{T}$ [$K^{-1}$]', fontsize=globalTextSize)
    ax.set_ylabel(r'$\ln{\tau}$ [$\ln{s}$]', fontsize=globalTextSize)
    ax.tick_params(labelsize=globalTextSize)
    ax.legend(fontsize=globalTextSize)
    
    return fig, p_fit
    
def printFittedParams(p_fit):
    """
    Prints the parameters that are in the dictionary p_fit
    with the names and uncertainties
    """
    
    print('\nThe fitted parameters are:')
    for n in range(len(p_fit['params'])):
        print('{} = {} +/- {}'.format(p_fit['quantities'][n], p_fit['params'][n], p_fit['sigmas'][n]))

def readPFITinOrder(p_fit, plotType='O'):

    p = []
    if plotType=='QT':
        tQT = p_fit['params'][p_fit['quantities'].index('tQT')]
        p = [tQT]
    elif plotType=='R':
        Cr = p_fit['params'][p_fit['quantities'].index('Cr')]
        n = p_fit['params'][p_fit['quantities'].index('n')]
        p = [Cr, n]
    elif plotType=='O':
        t0 = p_fit['params'][p_fit['quantities'].index('t0')]
        Ueff = p_fit['params'][p_fit['quantities'].index('Ueff')]
        p = [t0, Ueff]
    elif plotType=='QTR':
        p = readPFITinOrder(p_fit, plotType='QT') + readPFITinOrder(p_fit, plotType='R')
    elif plotType=='QTO':
        p = readPFITinOrder(p_fit, plotType='QT') + readPFITinOrder(p_fit, plotType='O')
    elif plotType=='RO':
        p = readPFITinOrder(p_fit, plotType='R') + readPFITinOrder(p_fit, plotType='O')
    elif plotType=='QTRO':
        p = readPFITinOrder(p_fit, plotType='QT') + readPFITinOrder(p_fit, plotType='R') + readPFITinOrder(p_fit, plotType='O')
    
    return p

def readTvsTauFile(fileName):
    
    D = np.loadtxt(fileName, skiprows=1)
    
    T = D[:,0]
    tau = D[:,1]
    
    return T, tau
    
def addPartialModel(fig, Tmin, Tmax, p_fit, plotType='O'):
    
    ax = fig.get_axes()[0]
    
    f = getFittingFunction(fitType=plotType)
    p = readPFITinOrder(p_fit, plotType=plotType)
    
    T_space = np.linspace(Tmin, Tmax, 101)
    line, = ax.plot(1/T_space, np.ones(T_space.shape)*f(T_space, *p), label=plotType)
    #ax.legend(fontsize=globalTextSize)
    
    return line

def readTwoColumnCSV(fileName, newFile, saveFile=False):

    D = np.loadtxt(fileName, delimiter=';')
    D = np.split(D, len(D[:,0])/4)
    
    T_recip, lntau = [], []
    for ary in D:
        ary = np.mean(ary, axis=0)
        T_recip.append(ary[0])
        lntau.append(ary[1])
    
    T = 1/np.array(T_recip)
    tau = np.exp(np.array(lntau))
    
    sort_indices = T.argsort()
    T = T[sort_indices]
    tau = tau[sort_indices]
    
    if saveFile:
        f = open('{}.dat'.format(fileName.split('.')[1]), 'w')
        for n in range(len(T)):
            f.write('{} {}\n'.format(T[n], tau[n]))
        f.close()
    
    return T, tau

def printUeffInKelvin(p_fit):
    """
    Reads the dictionary p_fit to see, whether an Orbach fit was made.
    If the fit was made, the effective barrier is printed in Kelvin.
    If the fit was not made, a message is printed telling that.
    """
    
    kB = sc.Boltzmann
    
    quantities = p_fit['quantities']
    
    if 'Ueff' in quantities:
        index = quantities.index('Ueff')
        Ueffval = p_fit['params'][index]/kB
        Ueffsigma = p_fit['sigmas'][index]/kB
        
        print('Ueff = {}K +/- {}K'.format(Ueffval, Ueffsigma))
    
    else:
        print('A Ueff-parameter has not been produced by the fit.')

def plotSusceptibility(D, T_plot, type='Xpp'):

    T = np.array(sorted([float(x[:-1]) for x in D.keys()]))
    index = np.argmin(np.abs(T-T_plot))
    T_to_use = T[index]
    
    fig = plt.figure()
    ax = fig.add_subplot(111)
    x,y = 0,0
    if type == 'Xpp':
        y = D['{}K'.format(T_to_use)]["X'' (emu/(Oe*mol))"]
        type = "''"
    elif type == 'Xp':
        y = D['{}K'.format(T_to_use)]["X' (emu/(Oe*mol))"]
        type = "'"
    x = D['{}K'.format(T_to_use)]['Frequency (Hz)']
    
    ax.semilogx(x,y, marker=globalPointMarker,
                     linestyle='None',
                     markerfacecolor=calcTcolor(T_to_use, 4.0, 25),
                     markeredgecolor='None',
                     label='{}K'.format(T_to_use))
    ax.set_title(r'$\chi${} vs frequency at {}K'.format(type, T_to_use), fontsize=globalTextSize)
    ax.set_xlabel(r"$\nu$ [$Hz$]", fontsize=globalTextSize)
    ax.set_ylabel(r'$\chi$'+'{}'.format(type)+r'[$\frac{emu}{Oe  mol}$]', fontsize=globalTextSize)
    ax.legend()
    
    return fig
    
if __name__ == '__main__':
    
    """
    PUT YOUR VALUES HERE
    """
    # Name of the file printed by the PPMS
    fileName = 'data\\dy_dbm\\20180209DyII_ac0.dat'
    
    # Molar weight of the sample
    molWeight = 988.42 # in g/mol
    
    # Mass of the sample that was used (in g)
    sampleMass = 17.8*10**-3 # in g
    
    # Mass of the Parafilm that was used (in mg)
    filmMass = 11.2

    # The temperature range in which the fit should be performed
    tempRange = [5.9, 10]

    # Whether to plot the result of CCFit on top of measured data
    plotFit = False

    # Whether to save the TvsTau file or not
    saveFile = True

    # Name of the TvsTau-file, if it is to be made
    TvsTaufileName = 'Dy_DBM_ac0Oe.dat'

    # Which type of fit should be made? (In order QT, R, O. For example to use quantum tunneling and Raman, put in 'QTR'. The opposite, namely 'RQT' will not work)
    fitToMake = 'O'
    
    D = readACDATAtoARRAY(fileName, dataOrigin='PPMS')
    D = calculateX(D, molWeight, sampleMass, filmMass, setup='capsule')
    prepareCCFITinput(D)
    D = runCCFIT(D)
    T, tau = makeTvsTauFile(D, TvsTaufileName, saveFile=saveFile)
    guess = getParameterGuesses(T, tau)
    
    #fig1 = ColeColePlot(D, addFit=plotFit)
    #fig2 = plotXppvsFreq(D, addFit=plotFit)
    fig3, p_fit = fitRelaxation(TvsTaufileName, tempRange, fitType=fitToMake)
    
    # The function addPartialModel can be used to add a plot showing the T vs tau simulation for a single process.
    # Use it as many times as deemed appropriate.
    # Parameters (in order):
    # <figure to plot on>,
    # <a list of temperatures>, 
    # <a list of relaxation times>,
    # <minimum temperature for simulation>, 
    # <maximum temperature for simulation>, 
    # <fitted parameters from the fitRelaxation-function>, 
    # <which type of plot to add (possibilities: 'QT' or 'R' or 'O'>
    
    # For example, to add a simulation of a Raman process to figure 3 between
    # 7 K and 18 K, use the line below
    #addPartialModel(fig3, T, tau, 7, 18, p_fit, plotType='R')
    #addPartialModel(fig3, T, tau, 18, 25, p_fit, plotType='O')
    
    # For example, to add a simulation of an Orbach process to figure 3 between
    # 15 K and 25 K, use the line below
    # addPartialModel(fig3, T, tau, 15, 25, p_fit, plotType='O')
    
    # KEEP THESE METHOD CALLS
    printFittedParams(p_fit)
    printUeffInKelvin(p_fit)
    ax = fig3.get_axes()[0]
    ax.legend()
    plt.show()














import ctypes
import sys
from PyQt5.QtWidgets import QApplication
from molmag_ac_gui.ac_gui import *

if __name__ == '__main__':
    
    myappid = 'AC Processing v1.0'
    ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)
    
    app = QApplication(sys.argv)
    w = ACGui()
    sys.exit(app.exec_())


from setuptools import setup, find_packages

with open('README.md') as readme_file:
    README = readme_file.read()

with open('HISTORY.md') as history_file:
    HISTORY = history_file.read()

setup_args = dict(
    name='molmag_ac_gui',
    version='0.1.8',
    description='A user interface and functions to work with magnetic relaxation',
    long_description_content_type="text/markdown",
    long_description=README + '\n\n' + HISTORY,
    license='MIT',
    packages=find_packages(),
    author='Emil A. Klahn',
    author_email='emil.klahn@gmail.com',
    keywords=['Magnetism', 'Molecular magnetism', 'Magnetic relaxation'],
    url='https://eandklahn.github.io',
    download_url='https://pypi.org/project/molmag-ac-gui'
)

install_requires = [
    'asteval==0.9.21',
    'cycler==0.10.0',
    'future==0.18.2',
    'kiwisolver==1.3.1',
    'lmfit==1.0.1',
    'matplotlib==3.3.3',
    'numpy==1.19.4',
    'pandas==1.1.4',
    'Pillow==8.0.1',
    'pyparsing==2.4.7',
    'PyQt5==5.15.2',
    'PyQt5-sip==12.8.1',
    'python-dateutil==2.8.1',
    'pytz==2020.4',
    'scipy==1.5.4',
    'six==1.15.0',
    'uncertainties==3.1.5'
]

if __name__ == '__main__':
    setup(**setup_args, install_requires=install_requires)


# PYTHON
import ctypes
import sys
import numpy as np
import pandas as pd
import time
from subprocess import Popen, PIPE
import re
# OWN ONES
from process_ac import *
# EXTERNAL
import scipy.constants as scicon
from scipy.optimize import minimize
import matplotlib.pyplot as plt
from matplotlib.figure import Figure
from matplotlib.backends.qt_compat import QtCore, QtWidgets
from lmfit import Parameters, minimize

from PyQt5.QtWinExtras import QWinTaskbarButton
from PyQt5.QtGui import QIcon, QFont, QDoubleValidator
from PyQt5.QtWidgets import (QMainWindow, QWidget, QApplication, QPushButton, QLabel, QAction, QComboBox, QStackedWidget,
                             QDoubleSpinBox, QFormLayout, QCheckBox, QVBoxLayout, QMessageBox, QSplitter, QGridLayout,
                             QHBoxLayout, QFileDialog, QDialog, QLineEdit, QListWidget, QListWidgetItem, QTabWidget,
                             QScrollArea)
if int(QtCore.qVersion().split('.')[0])>=5:
    from matplotlib.backends.backend_qt5agg import (FigureCanvas, NavigationToolbar2QT as NavigationToolbar)
#print('PyQt version is {}'.format(QtCore.qVersion()))

class ACGui(QMainWindow):

    def __init__(self):
    
        super().__init__()
        
        self.initUI()
        
    def initUI(self):
        
        """About"""
        self.about_information = {'author': 'Emil A. Klahn (eklahn@chem.au.dk)',
                                  'webpage': 'https://chem.au.dk/en/molmag',
                                  'personal': 'https://eandklahn.github.io'}
        
        self.startUp = True
        
        """ Things to do with how the window is shown """
        self.setWindowTitle('AC Processing')
        self.setWindowIcon(QIcon('double_well_potential_R6p_icon.ico'))
        
        """ FONT STUFF """
        self.headline_font = QFont()
        self.headline_font.setBold(True)
        
        """ Setting up the main tab widget """
        self.all_the_tabs = QTabWidget()
        self.setCentralWidget(self.all_the_tabs)
        
        """ Constructing the data analysis tab ---------------------------------------------------------------------------------- """
        # Data containers for analysis
        self.data_file_origin = None
        self.data_T = None
        self.data_tau = None
        
        self.data_used_pointer = None
        self.data_not_used_pointer = None
        self.plot_of_fit_pointer = None
        
        self.used_T = None
        self.not_used_T = None
        
        self.used_tau = None
        self.not_used_tau = None
        
        self.fitted_parameters = None
        self.used_indices = None
        
        self.simulation_items = []
        
        # Data containers for treatment
        
        self.raw_df = None
        self.raw_df_origin = None
        self.meas_info = {}
        self.data_header_idx = 0
        self.num_meas_freqs = 0
        self.num_meas_temps = 0
        self.temp_subsets = []
        self.meas_temps = []
        self.Xd_capsule = -1.8*10**-8 # unit: emu/Oe
        self.Xd_film = 6.47*10**-10 # unit: emu/(Oe*mg)
        self.data_names = {'freq': 'Frequency (Hz)',
                           'amplitude': 'Amplitude (Oe)',
                           'mag_field': 'Magnetic Field (Oe)',
                           'Xp': "X' (emu/(Oe*mol))",
                           'Xpp': "X'' (emu/(Oe*mol))"}
        
        self.raw_data_fit = None
        
        """ Creating data analysis tab ----------------------------------------------------- """
        self.data_analysis_tab = QSplitter()
        self.all_the_tabs.addTab(self.data_analysis_tab, 'Analysis')
        
        ## Adding load controls
        self.load_wdgt = QWidget()
        self.load_layout = QVBoxLayout()
        self.load_layout.addStretch()
        
        self.see_fit_btn = QPushButton('Fitted params')
        self.see_fit_btn.clicked.connect(self.print_fitted_params)
        self.load_layout.addWidget(self.see_fit_btn)
        
        self.load_btn = QPushButton('Load')
        self.load_btn.clicked.connect(self.load_t_tau_data)
        self.load_layout.addWidget(self.load_btn)
        
        self.load_wdgt.setLayout(self.load_layout)
        
        """ Adding plotting controls """
        self.ana_plot = PlottingWindow()
        self.ana_plot.ax.set_xlabel('Temperature [$K^{-1}$]')
        self.ana_plot.ax.set_ylabel(r'$\ln{\tau}$ [$\ln{s}$]')
        
        # Adding fit controls
        self.fit_wdgt = QWidget()
        self.fit_layout = QVBoxLayout()
        
        self.cb_headline = QLabel('Fit types to consider')
        self.cb_headline.setFont(self.headline_font)
        self.fit_layout.addWidget(self.cb_headline)
        
        self.orbach_cb = QCheckBox('Orbach')
        self.orbach_cb.stateChanged.connect(self.read_fit_type_cbs)
        self.fit_layout.addWidget(self.orbach_cb)
        
        self.raman_cb = QCheckBox('Raman')
        self.raman_cb.stateChanged.connect(self.read_fit_type_cbs)
        self.fit_layout.addWidget(self.raman_cb)
        
        self.qt_cb = QCheckBox('QT')
        self.qt_cb.stateChanged.connect(self.read_fit_type_cbs)
        self.fit_layout.addWidget(self.qt_cb)
        
        # Adding temperature controls
        self.temp_headline = QLabel('Temperature interval')
        self.temp_headline.setFont(self.headline_font)
        self.fit_layout.addWidget(self.temp_headline)
        
        self.temp_horizontal_layout = QHBoxLayout()
        self.temp_line = [QLabel('('), QDoubleSpinBox(), QLabel(','), QDoubleSpinBox(), QLabel(')')]
        
        self.temp_line[1].setRange(0,self.temp_line[3].value())
        self.temp_line[1].setSingleStep(0.1)
        self.temp_line[3].setRange(self.temp_line[1].value(),1000)
        self.temp_line[3].setSingleStep(0.1)
        
        self.temp_line[1].editingFinished.connect(self.set_new_temp_ranges)
        self.temp_line[3].editingFinished.connect(self.set_new_temp_ranges)
        for w in self.temp_line:
            self.temp_horizontal_layout.addWidget(w)
        
        self.fit_layout.addLayout(self.temp_horizontal_layout)
        
        # Adding a button to run a fit
        self.run_fit_btn = QPushButton('Run fit!')
        self.run_fit_btn.clicked.connect(self.make_the_fit)
        self.fit_layout.addWidget(self.run_fit_btn)
        
        # Adding a list to hold information about simulations
        self.simulations_headline = QLabel('Simulations')
        self.simulations_headline.setFont(self.headline_font)
        self.fit_layout.addWidget(self.simulations_headline)
        
        self.list_of_simulations = QListWidget()
        self.list_of_simulations.doubleClicked.connect(self.edit_a_simulation)
        self.fit_layout.addWidget(self.list_of_simulations)
        
        # Adding buttons to control simulation list
        self.sim_btn_layout = QHBoxLayout()
        
        self.delete_sim_btn = QPushButton('Delete')
        self.delete_sim_btn.clicked.connect(self.delete_sim)
        self.sim_btn_layout.addWidget(self.delete_sim_btn)
        
        self.edit_sim_btn = QPushButton('Edit')
        self.edit_sim_btn.clicked.connect(self.edit_a_simulation)
        self.sim_btn_layout.addWidget(self.edit_sim_btn)
        
        self.new_sim_btn = QPushButton('New')
        self.new_sim_btn.clicked.connect(self.add_new_simulation)
        self.sim_btn_layout.addWidget(self.new_sim_btn)
        
        self.fit_layout.addLayout(self.sim_btn_layout)
        
        self.fit_wdgt.setLayout(self.fit_layout)
        
        # Finalizing layout of the data analysis tab
        self.data_analysis_tab.addWidget(self.load_wdgt)
        self.data_analysis_tab.addWidget(self.ana_plot)
        self.data_analysis_tab.addWidget(self.fit_wdgt)
        
        """ Creating the data treatment tab  ------------------------------------------------------------------------------- """
        self.data_treatment_tab = QSplitter()
        self.all_the_tabs.addTab(self.data_treatment_tab, 'Data treatment')
        
        ### Making the left column (data loading and visualization controls)
        self.data_loading_wdgt = QWidget()
        self.data_layout = QVBoxLayout()
        
        self.raw_data_load_btn = QPushButton('(1) Load')
        self.raw_data_load_btn.clicked.connect(self.load_ppms_data)
        self.data_layout.addWidget(self.raw_data_load_btn)
        
        self.calculate_Xp_Xpp_btn = QPushButton("(2) Calc. X', X''")
        self.calculate_Xp_Xpp_btn.clicked.connect(self.calculate_Xp_and_Xpp)
        self.data_layout.addWidget(self.calculate_Xp_Xpp_btn)
        
        self.fit_Xp_Xpp_btn = QPushButton("(3) Fit X', X''")
        #self.fit_Xp_Xpp_btn.clicked.connect(self.fit_Xp_Xpp_w_ccfit)
        self.fit_Xp_Xpp_btn.clicked.connect(self.fit_Xp_Xpp_standalone)
        self.data_layout.addWidget(self.fit_Xp_Xpp_btn)
        
        self.copy_fit_to_ana_btn = QPushButton('(4) Copy for analysis')
        self.copy_fit_to_ana_btn.clicked.connect(self.copy_fit_to_analysis)
        self.data_layout.addWidget(self.copy_fit_to_ana_btn)
        
        self.save_fit_to_file_btn = QPushButton('(5) Save fit to file')
        self.save_fit_to_file_btn.clicked.connect(self.save_fit_to_file)
        self.data_layout.addWidget(self.save_fit_to_file_btn)
        
        ## Constructing data plotting layout
        self.raw_data_plot_lo = QVBoxLayout()
        
        self.analysis_plot_type_header = QLabel('Axis content')
        self.analysis_plot_type_header.setFont(self.headline_font)
        self.raw_data_plot_lo.addWidget(self.analysis_plot_type_header)
        
        self.analysis_plot_type_combo = QComboBox()
        self.analysis_plot_type_combo.addItems(['Raw data', 'Fitted'])
        self.analysis_plot_type_combo.currentIndexChanged.connect(self.switch_analysis_view)
        self.raw_data_plot_lo.addWidget(self.analysis_plot_type_combo)
        
        self.raw_data_plot_header = QLabel('Raw data plotting')
        self.raw_data_plot_header.setFont(self.headline_font)
        self.raw_data_plot_lo.addWidget(self.raw_data_plot_header)
        
        # Constructing the x combobox
        self.data_ana_x_lo = QHBoxLayout()
        self.analysis_x_combo_lbl = QLabel('x')
        self.data_ana_x_lo.addWidget(self.analysis_x_combo_lbl)
        
        self.analysis_x_combo = QComboBox()
        self.analysis_x_combo.currentIndexChanged.connect(self.plot_from_combo)
        self.data_ana_x_lo.addWidget(self.analysis_x_combo)
        self.raw_data_plot_lo.addLayout(self.data_ana_x_lo)
        
        # Constructing the y combobox
        self.data_ana_y_lo = QHBoxLayout()
        self.analysis_y_combo_lbl = QLabel('y')
        self.data_ana_y_lo.addWidget(self.analysis_y_combo_lbl)
        
        self.analysis_y_combo = QComboBox()
        self.analysis_y_combo.currentIndexChanged.connect(self.plot_from_combo)
        self.data_ana_y_lo.addWidget(self.analysis_y_combo)
        self.raw_data_plot_lo.addLayout(self.data_ana_y_lo)
        
        # Constructing a combobox for plotting fitted data
        self.fitted_data_plot_header = QLabel('Fit data plotting')
        self.fitted_data_plot_header.setFont(self.headline_font)
        self.raw_data_plot_lo.addWidget(self.fitted_data_plot_header)
        
        self.fit_data_plot_combo = QComboBox()
        self.fit_data_plot_combo.addItems(['ColeCole', 'FreqVSXpp'])
        self.fit_data_plot_combo.currentIndexChanged.connect(self.plot_from_itemlist)
        self.raw_data_plot_lo.addWidget(self.fit_data_plot_combo)
        
        self.data_layout.addLayout(self.raw_data_plot_lo)
        
        ## Finalizing the data loading widget
        self.data_layout.addStretch()
        self.data_loading_wdgt.setLayout(self.data_layout)
        
        # Making the middle of the tab (data visualization)
        self.treat_sw = QStackedWidget()
        
        self.treat_raw_plot = PlottingWindow()
        self.treat_fit_plot = PlottingWindow()
        
        self.treat_sw.addWidget(self.treat_raw_plot)
        self.treat_sw.addWidget(self.treat_fit_plot)
        
        ## Making the right column (parameter controls)
        self.param_wdgt = QWidget()
        self.param_layout = QVBoxLayout()
        
        # Sample mass
        self.sample_info_layout = QVBoxLayout()
        
        self.sample_info_header = QLabel('Sample information')
        self.sample_info_header.setFont(self.headline_font)
        self.sample_info_layout.addWidget(self.sample_info_header)
        
        # Sample mass edit
        self.sample_mass_layout = QHBoxLayout()
        self.sample_info_layout.addLayout(self.sample_mass_layout)
        
        self.sample_mass_lbl = QLabel('m (sample) [mg]')
        self.sample_mass_layout.addWidget(self.sample_mass_lbl)
        
        self.sample_mass_inp = QLineEdit()
        self.sample_mass_inp.setValidator(QDoubleValidator())
        self.sample_mass_layout.addWidget(self.sample_mass_inp)
        
        # Film mass edit
        self.film_mass_layout = QHBoxLayout()
        self.sample_info_layout.addLayout(self.film_mass_layout)
        
        self.film_mass_lbl = QLabel('m (film) [mg]')
        self.film_mass_layout.addWidget(self.film_mass_lbl)
             
        self.film_mass_inp = QLineEdit()
        self.film_mass_inp.setValidator(QDoubleValidator())
        self.film_mass_layout.addWidget(self.film_mass_inp)
        
        # Molar mass edit
        self.molar_mass_lo = QHBoxLayout()
        self.sample_info_layout.addLayout(self.molar_mass_lo)
        
        self.molar_mass_lbl = QLabel('M [g/mol]')
        self.molar_mass_lo.addWidget(self.molar_mass_lbl)
        
        self.molar_mass_inp = QLineEdit()
        self.molar_mass_inp.setValidator(QDoubleValidator())
        self.molar_mass_lo.addWidget(self.molar_mass_inp)
        
        # Sample X' edit
        self.sample_xd_lo = QHBoxLayout()
        self.sample_info_layout.addLayout(self.sample_xd_lo)
        
        self.sample_xd_lbl = QLabel(u"X\u1D05"+' (sample) [emu/(Oe*mol)]')
        self.sample_xd_lo.addWidget(self.sample_xd_lbl)
        
        self.sample_xd_inp = QLineEdit()
        self.sample_xd_inp.setText('-6e-7')
        self.sample_xd_inp.setValidator(QDoubleValidator())
        self.sample_xd_lo.addWidget(self.sample_xd_inp)
        
        # Mass load button
        self.load_mass_lo = QHBoxLayout()
        self.sample_info_layout.addLayout(self.load_mass_lo)
        
        self.load_mass_btn = QPushButton('Load mass from file')
        self.load_mass_btn.clicked.connect(self.load_sample_film_mass)
        self.load_mass_lo.addWidget(self.load_mass_btn)
        self.load_mass_lo.addStretch()
        
        self.param_layout.addLayout(self.sample_info_layout)
        
        # List of fitted raw data
        self.treat_raw_fit_headline = QLabel('Fitted parameters')
        self.treat_raw_fit_headline.setFont(self.headline_font)
        self.param_layout.addWidget(self.treat_raw_fit_headline)
        
        self.treat_raw_fit_list = QListWidget()
        self.param_layout.addWidget(self.treat_raw_fit_list)
        self.treat_raw_fit_list.doubleClicked.connect(self.update_raw_plot_status)
        
        ## Finalizing layout
        
        self.param_layout.addStretch()
        self.param_wdgt.setLayout(self.param_layout)
        
        self.data_treatment_tab.addWidget(self.data_loading_wdgt)
        self.data_treatment_tab.addWidget(self.treat_sw)
        self.data_treatment_tab.addWidget(self.param_wdgt)
        
        """ Making a menubar -------------------------------------------------------------- """
        self.menu_bar = self.menuBar()
        
        # File menu
        self.file_menu = self.menu_bar.addMenu('File')
        
        self.quit_action = QAction('&Quit', self)
        self.quit_action.setShortcut("Ctrl+Q")
        self.quit_action.triggered.connect(sys.exit)
        self.file_menu.addAction(self.quit_action)
        
        # Simulation menu
        self.sim_menu = self.menu_bar.addMenu('Simulation')
        
        self.add_sim_w_menu = QAction('&New', self)
        self.add_sim_w_menu.setShortcut("Ctrl+Shift+N")
        self.add_sim_w_menu.triggered.connect(self.add_new_simulation)
        self.sim_menu.addAction(self.add_sim_w_menu)
        
        # About menu
        self.help_menu = self.menu_bar.addMenu('Help')
        
        self.help_about_menu = QAction('About', self)
        self.help_about_menu.triggered.connect(self.show_about_dialog)
        self.help_menu.addAction(self.help_about_menu)
        
        # Showing the GUI
        self.load_t_tau_data()
        
        self.show()
    
    def show_about_dialog(self):
    
        w = AboutDialog(info=self.about_information)
        w.exec_()
    
    def copy_fit_to_analysis(self):
    
        try:
            self.set_new_t_tau(np.array(self.meas_temps),
                               np.array(self.raw_data_fit['Tau']))
            self.read_indices_for_used_temps()
            self.plot_t_tau_on_axes()
        except TypeError:
            print('When the fitted data does not yet exist')
        
    
    def get_ccfit_starting_params(self):
        """Reimplementation of CCFITStartingGuesses from process_ac"""
        
        lowest_t_idx = self.meas_temps.argmin()
        
        v = self.temp_subsets[lowest_t_idx][self.data_names['freq']]
        Xp = self.temp_subsets[lowest_t_idx][self.data_names['Xp']]
        Xpp = self.temp_subsets[lowest_t_idx][self.data_names['Xpp']]
        
        tau = 1/(2*np.pi*v[Xpp.idxmax()])
        Xs = 0
        Xt = Xp[0]
        alpha = 0.1
        
        return (Xs, Xt, tau, alpha)
    
    def write_file_for_ccfit(self):
        """
        This function is deprecated since 27/11/20. The fitting is now done internally.
        """
        f = open('ccin.dat', 'w')
        f.write('1 {} {}\n'.format(self.num_meas_temps, self.num_meas_freqs))
        f.write('{} {} {} {}\n'.format(*self.get_ccfit_starting_params()))
        
        for n in range(self.num_meas_freqs):
            
            line = [self.raw_df[self.data_names['freq']].iloc[n]]
            
            for i in range(self.num_meas_temps):
                line += [self.raw_df[self.data_names['Xp']][i*self.num_meas_freqs+n],
                         self.raw_df[self.data_names['Xpp']][i*self.num_meas_freqs+n]]
                
            line.append('\n')
            line = ' '.join(str(e) for e in line)
            
            f.write(line)

        f.close()
        
    def run_ccfit(self):
        """
        This function is deprecated since 27/11/20. The fitting is now done internally.
        """
        ccfit = Popen('cc-fit ccin.dat', stdin=PIPE, stdout=PIPE, stderr=PIPE)
        ccfit.stdin.write(b'\n')
        out, err = ccfit.communicate()
        print('Using cc-fit')
        
    def switch_analysis_view(self):
        
        idx = self.analysis_plot_type_combo.currentIndex()
        self.treat_sw.setCurrentIndex(idx)
        
    def read_ccfit_output(self):
        """
        This function is deprecated since 27/11/20. The fitting is now done internally.
        """
        filename = 'ccin.dat_cc-fit.out'
        headers = self.get_single_line(filename, 13).split()
        self.raw_data_fit = pd.read_csv('ccin.dat_cc-fit.out',
                                        names=headers,
                                        sep='   ',
                                        skiprows=13,
                                        engine='python')
        self.update_treat_raw_fit_list()
        self.plot_from_itemlist()
    
    def update_treat_raw_fit_list(self):
        
        self.treat_raw_fit_list.clear()
        
        for i in range(self.num_meas_temps):
            newitem = QListWidgetItem()
            newitem.setText('{}, {}, {}'.format(self.meas_temps[i],True, True))
            plotting_dict = {'temp': self.meas_temps[i],
                             'raw': True,
                             'fit': True}
            newitem.setData(32, plotting_dict)
            self.treat_raw_fit_list.addItem(newitem)
    
    def fit_Xp_Xpp_standalone(self):
        
        #### Defining the objective function for lmfit ####
        def objective(params, v, data):
    
            Xp_data = data[0]
            Xpp_data = data[1]
            
            Xp_model = Xp_dataset(params, v)
            Xpp_model = Xpp_dataset(params, v)
            
            Xp_resid = Xp_data - Xp_model
            Xpp_resid = Xpp_data - Xpp_model
    
            return np.concatenate([Xp_resid, Xpp_resid])
        ########
        
        T, Xs, Xt, tau, alpha, resid = [],[],[],[],[],[]
        if self.data_names['Xp'] in self.raw_df.columns:
            for t_idx in range(self.num_meas_temps):
                v = np.array(self.temp_subsets[t_idx][self.data_names['freq']])
                Xp = np.array(self.temp_subsets[t_idx][self.data_names['Xp']])
                Xpp = np.array(self.temp_subsets[t_idx][self.data_names['Xpp']])
                
                data = [Xp, Xpp]
                
                tau_init = (v[np.argmax(Xpp)]*2*np.pi)**-1
                fit_params = Parameters()
                fit_params.add('Xs', value=Xp[-1], min=0, max=np.inf)
                fit_params.add('Xt', value=Xp[0], min=0, max=np.inf)
                fit_params.add('tau', value=tau_init, min=0, max=np.inf)
                fit_params.add('alpha', value=0.1, min=0, max=np.inf)
                
                out = minimize(objective, fit_params, args=(v, data))
                T.append(self.meas_temps[t_idx])
                Xs.append(out.params['Xs'].value)
                Xt.append(out.params['Xt'].value)
                tau.append(out.params['tau'].value)
                alpha.append(out.params['alpha'].value)
                resid.append(out.residual.sum())
        fit_result = pd.DataFrame(data={'Temp': T,
                                        'ChiS': Xs,
                                        'ChiT': Xt,
                                        'Tau': tau,
                                        'Alpha': alpha,
                                        'Residual': resid})    
        
        self.raw_data_fit = fit_result
        self.update_treat_raw_fit_list()
        self.plot_from_itemlist()
        set_idx = self.analysis_plot_type_combo.findText('Fitted')
        self.analysis_plot_type_combo.setCurrentIndex(set_idx)    
        
    def fit_Xp_Xpp_w_ccfit(self):
        """
        This function is deprecated since 27/11/20. Fitting is now done internally instead.
        """
        if self.data_names['Xp'] in self.raw_df.columns:
            working_directory = os.path.dirname(self.raw_df_origin)
            os.chdir(working_directory)
            self.write_file_for_ccfit()
            self.run_ccfit()
            self.read_ccfit_output()
        else:
            print('Give messagebox that they dont exist')
    
    def get_single_line(self, filename, line_number):

        f = open(filename, 'r')
        for n in range(line_number):
            line = f.readline()
        f.close()
        
        return line
    
    def save_fit_to_file(self):
        
        name = QFileDialog.getSaveFileName(self, 'Save File')
        print(name)
        if self.raw_data_fit is None:
            msg = QMessageBox()
            msg.setText('There is nothing to save')
            msg.setDetailedText("""There is probably no fit yet...""")
            msg.exec_()
        elif name=='':
            pass
            print('No file selected')
        else:
            self.raw_data_fit.to_csv(name[0]+'.txt', sep=';')
    
    def load_sample_film_mass(self):
    
        starting_directory = os.getcwd()
        filename_info = QFileDialog().getOpenFileName(self, 'Open file', starting_directory)
        filename = filename_info[0]
        
        if filename == '':
            return 0
        else:
            sample = self.get_single_line(filename, 9).strip()
            film = self.get_single_line(filename, 10).strip()
        
        try:
            sample = sample.split(',')
            film = film.split(',')
            
            print(sample)
            print(film)
            
            assert sample[0] == 'INFO' and (sample[1] == 's' or sample[1] == 'sample')
            assert film[0] == 'INFO' and (film[1] == 'f' or film[1] == 'film')
        except AssertionError:
            msg = QMessageBox()
            msg.setText('File not as expected')
            msg.setDetailedText("""Expected:
line 9: INFO,s,<mass>mg
line 10: INFO,f,<mass>mg""")
            msg.exec_()
        else:
            sample = float(sample[2][:-2])
            film = float(film[2][:-2])
            
            self.sample_mass_inp.setText(str(sample))
            self.film_mass_inp.setText(str(film))
    
    def calculate_Xp_and_Xpp(self):
        
        #print(self.data_names['VERSION'])
        #print(self.data_names['BUILD'])
        
        if self.raw_df is None:
            # Don't do the calculation, if there is nothing to calculate on
            pass
        elif "X'' (emu/(Oe*mol))" in self.raw_df.columns:
            # Don't add an element that is already there
            pass
        elif "AC X'  (emu/Oe)" in self.raw_df.columns:
            # Data was read from source where the column already exists.
            # Update data names
            self.data_names.update({'Xp': "AC X'  (emu/Oe)",
                                    'Xpp': 'AC X" (emu/Oe)'})
        else:
            try:
                sample_mass = float(self.sample_mass_inp.text())
                film_mass = float(self.film_mass_inp.text())
                molar_mass = float(self.molar_mass_inp.text())
                Xd_sample = float(self.sample_xd_inp.text())
            except ValueError as error:
                msg = QMessageBox()
                msg.setWindowTitle('Error')
                msg.setText('Could not read sample information')
                msg.exec_()
            else:
                H = self.raw_df[self.data_names['amplitude']]
                H0 = self.raw_df[self.data_names['mag_field']]
                Mp = self.raw_df["M' (emu)"]
                Mpp = self.raw_df["M'' (emu)"]
                
                Xp = (Mp - self.Xd_capsule*H - self.Xd_film*film_mass*H)*molar_mass/(sample_mass*H) - Xd_sample*molar_mass
                Xpp = Mpp/(sample_mass*H)*molar_mass
                
                Xp_idx = self.raw_df.columns.get_loc("M' (emu)")+1
                self.raw_df.insert(Xp_idx, column="X' (emu/(Oe*mol))", value=Xp)
                
                Xpp_idx = self.raw_df.columns.get_loc("M'' (emu)")+1
                self.raw_df.insert(Xpp_idx, column="X'' (emu/(Oe*mol))", value=Xpp)
        
        self.update_temp_subsets()
        self.update_analysis_combos()
    
    def update_itemdict(self, item, itemdict):
        
        item.setData(32, itemdict)
        item.setText('{}, {}, {}'.format(itemdict['temp'],
                                         itemdict['raw'],
                                         itemdict['fit']))
    
    def update_raw_plot_status(self):
        
        w = FitResultPlotStatus(list_input=self.treat_raw_fit_list)
        finished_value = w.exec_()
        if not finished_value:
            pass
        else:
            final_states = w.checked_items
            
            for i, boxes in enumerate(final_states):
                item = self.treat_raw_fit_list.item(i)
                item_data = item.data(32)
                item_data['raw'] = boxes[0].isChecked()
                item_data['fit'] = boxes[1].isChecked()
                self.update_itemdict(item, item_data)
            
        self.plot_from_itemlist()
        self.treat_fit_plot.canvas.draw()
    
    def plot_from_itemlist(self):
        
        if self.treat_raw_fit_list.count()==0:
            return
        else:
            self.treat_fit_plot.ax.clear()
            plot_type = self.fit_data_plot_combo.currentText()
            
            if plot_type == 'FreqVSXpp':
                x_name = self.data_names['freq']
                y_name = self.data_names['Xpp']
                for row in range(self.num_meas_temps):
                    
                    item = self.treat_raw_fit_list.item(row)
                    itemdict = item.data(32)
                    
                    if itemdict['raw']:
                        self.treat_fit_plot.ax.plot(self.temp_subsets[row][x_name],
                                                    self.temp_subsets[row][y_name],
                                                    'ko')
                    if itemdict['fit']:
                        self.treat_fit_plot.ax.plot(self.temp_subsets[row][x_name],
                                                    Xpp_(self.temp_subsets[row][self.data_names['freq']],
                                                        self.raw_data_fit['ChiS'].iloc[row],
                                                        self.raw_data_fit['ChiT'].iloc[row],
                                                        self.raw_data_fit['Tau'].iloc[row],
                                                        self.raw_data_fit['Alpha'].iloc[row]),
                                                    c=calcTcolor(self.meas_temps[row],
                                                                self.meas_temps[0],
                                                                self.meas_temps[-1]))
                self.treat_fit_plot.ax.set_xscale('log')
                self.treat_fit_plot.ax.set_xlabel(x_name)
                self.treat_fit_plot.ax.set_ylabel(y_name)
        
            elif plot_type == 'ColeCole':
                x_name = self.data_names['Xp']
                y_name = self.data_names['Xpp']
                for row in range(self.num_meas_temps):
                    
                    item = self.treat_raw_fit_list.item(row)
                    itemdict = item.data(32)
                    
                    if itemdict['raw']:
                        self.treat_fit_plot.ax.plot(self.temp_subsets[row][x_name],
                                                self.temp_subsets[row][y_name],
                                                'ko')
                    if itemdict['fit']:
                        self.treat_fit_plot.ax.plot(Xp_(self.temp_subsets[row][self.data_names['freq']],
                                                        self.raw_data_fit['ChiS'].iloc[row],
                                                        self.raw_data_fit['ChiT'].iloc[row],
                                                        self.raw_data_fit['Tau'].iloc[row],
                                                        self.raw_data_fit['Alpha'].iloc[row]),
                                                    Xpp_(self.temp_subsets[row][self.data_names['freq']],
                                                        self.raw_data_fit['ChiS'].iloc[row],
                                                        self.raw_data_fit['ChiT'].iloc[row],
                                                        self.raw_data_fit['Tau'].iloc[row],
                                                        self.raw_data_fit['Alpha'].iloc[row]),
                                                    c=calcTcolor(self.meas_temps[row],
                                                                self.meas_temps[0],
                                                                self.meas_temps[-1]))
                self.treat_fit_plot.ax.set_xlabel(x_name)
                self.treat_fit_plot.ax.set_ylabel(y_name)
            
            self.treat_fit_plot.canvas.draw()
        
    def plot_from_combo(self):
        
        self.treat_raw_plot.ax.clear()
        
        idx_x = self.analysis_x_combo.currentIndex()
        idx_y = self.analysis_y_combo.currentIndex()
        
        x_label = self.raw_df.columns[idx_x]
        y_label = self.raw_df.columns[idx_y]
        
        self.treat_raw_plot.ax.plot(self.raw_df[x_label],
                                    self.raw_df[y_label],
                                    'ko-',
                                    linewidth=2)
        self.treat_raw_plot.ax.set_xlabel(x_label)
        self.treat_raw_plot.ax.set_ylabel(y_label)
        
        self.treat_raw_plot.canvas.draw()
    
    def update_data_names(self):
        
        with open(self.ppms_data_file, 'r') as f:
            f_content = f.read()
            
        VERSION_PATTERN = r'INFO,PPMS ACMS\s(II\s)*.*\s.*\s(?P<VERSION>.*)\sBuild\s(?P<BUILD>[1-9]{1,2}),APPNAME'
        m = re.search(VERSION_PATTERN, f_content)
        VERSION, BUILD = m.group('VERSION'), m.group('BUILD')
        
        if (VERSION, BUILD) == ('1.0.9', '14'):
            self.data_names.update({'freq': 'Frequency (Hz)',
                                    'amplitude': 'Amplitude (Oe)',
                                    'mag_field': 'Magnetic Field (Oe)'})
        elif (VERSION, BUILD) == ('1.0.8', '33'):
            self.data_names.update({'freq': 'AC Frequency (Hz)',
                                    'amplitude': 'AC Drive (Oe)',
                                    'mag_field': 'Magnetic Field (Oe)'})
        
        self.data_names.update({'VERSION': VERSION,
                                'BUILD': BUILD})
                
    def load_ppms_data(self):
    
        starting_directory = os.getcwd()
        filename_info = QFileDialog().getOpenFileName(self, 'Open file', starting_directory)
        filename = filename_info[0]
        
        header_start = 0
        header_lines = []
        
        if filename == '':
            pass
        else:
            self.ppms_data_file = filename
        try:
            with open(self.ppms_data_file, 'r') as f:
                f_content = f.readlines()
            for i, line in enumerate(f_content):
                if '[Header]' in line:
                    header_start = i+1
                if '[Data]' in line:
                    self.data_header_idx = i+1
                    header_lines = f_content[header_start:self.data_header_idx]
                    break
            
            self.raw_df = pd.read_csv(filename,
                                      header=self.data_header_idx,
                                      engine='python')
        except Exception as e:
            pass
            print(e)
        else:
            self.cleanup_loaded_ppms()
            self.update_data_names()
            self.num_meas_freqs = len(set(self.raw_df[self.data_names['freq']]))
            #print('freqs', self.num_meas_freqs)
            #print(set(self.raw_df[self.data_names['freq']]))
            self.num_meas_temps = int(self.raw_df.shape[0]/self.num_meas_freqs)
            #print('temps', self.num_meas_temps)
            self.raw_df_origin = filename
            self.update_temp_subsets()
            self.update_meas_temps()
            self.update_analysis_combos()
    
    def update_meas_temps(self):
        
        meas_temps = []
        for sub in self.temp_subsets:
            meas_temps.append(sub['Temperature (K)'].mean())
        self.meas_temps = np.array(meas_temps)
    
    def cleanup_loaded_ppms(self):
        
        rows_to_remove = []
        # Removing instrument comment lines (for ACMS version 1.0.8, build 33)
        for i, val in enumerate(self.raw_df['Comment']):
            if not pd.isnull(val):
                rows_to_remove.append(i-1)
                rows_to_remove.append(i)
        self.raw_df.drop(rows_to_remove, inplace=True)
        
        # Make sure that the rows are named continuously
        old_indices = self.raw_df.index.values
        new_indices = list(range(len(old_indices)))
        self.raw_df.rename(index=dict(zip(old_indices, new_indices)),
                           inplace=True)
        
        # Removing NaN-columns
        for h in self.raw_df.columns:
            if pd.isnull(self.raw_df[h]).all():
                self.raw_df.drop(h, axis=1, inplace=True)
    
    def update_temp_subsets(self):
        
        self.temp_subsets = []
        nms = self.num_meas_freqs
        for n in range(self.num_meas_temps):
            self.temp_subsets.append(self.raw_df.iloc[n*nms:n*nms+nms])
       
    def update_analysis_combos(self):
    
        self.analysis_x_combo.clear()
        self.analysis_x_combo.addItems(self.raw_df.columns)
        
        self.analysis_y_combo.clear()
        self.analysis_y_combo.addItems(self.raw_df.columns)
    
    def print_fitted_params(self):
        
        if self.fitted_parameters == None:
            pass
        else:
            dialog = ParamDialog(param_dict=self.fitted_parameters)
            finished = dialog.exec_()
    
    def add_new_simulation(self):
    
        sim_dialog = SimulationDialog(fitted_parameters=self.fitted_parameters,
                                      plot_type_list=[],
                                      plot_parameters={'tQT': 0.01, 'Cr': 0.00, 'n': 0.00, 't0': 0.00, 'Ueff': 0.00},
                                      min_and_max_temps=[0,0])
        
        finished_value = sim_dialog.exec_()
        
        if finished_value:
            
            plot_type = sim_dialog.plot_type_list
            if len(plot_type)<1:
                pass
            else:
                p_fit = sim_dialog.plot_parameters
                T_vals = sim_dialog.min_and_max_temps
                
                plot_to_make = ''.join(plot_type)
                new_item_text = '{}, ({},{}), tQT: {}, Cr: {}, n: {}, t0: {}, Ueff: {}'.format(
                                plot_type, T_vals[0], T_vals[1], p_fit['tQT'], p_fit['Cr'],
                                p_fit['n'], p_fit['t0'], p_fit['Ueff'])
                
                new_list_item = QListWidgetItem()
                
                line = addPartialModel(self.ana_plot.fig,
                                        T_vals[0],
                                        T_vals[1],
                                        self.prepare_sim_dict_for_plotting(p_fit),
                                        plotType=plot_to_make)
                                        
                list_item_data = {'plot_type': plot_type,
                                  'p_fit': p_fit,
                                  'T_vals': T_vals,
                                  'line': line}
                
                self.list_of_simulations.addItem(new_list_item)
                new_list_item.setText(new_item_text)
                new_list_item.setData(32, list_item_data)
                
                self.ana_plot.canvas.draw()
                
        else:
            pass
    
    def edit_a_simulation(self):
        
        try:
            sim_item = self.list_of_simulations.selectedItems()[0]
        except IndexError:
            pass
        else:
            # Reading off information from the selected item
            old_data = sim_item.data(32)
            old_plot_type_input = old_data['plot_type']
            old_p_fit = old_data['p_fit']
            old_T_vals = old_data['T_vals']
            old_line = old_data['line']
            
            
            
            # Opening simulation dialog with old parameters
            sim_dialog = SimulationDialog(fitted_parameters=self.fitted_parameters,
                                          plot_type_list=old_plot_type_input,
                                          plot_parameters=old_p_fit,
                                          min_and_max_temps=old_T_vals)
            
            finished_value = sim_dialog.exec_()
            
            if finished_value:
                # Reading new parameters of simulation
                new_plot_type = sim_dialog.plot_type_list
                
                if len(new_plot_type)<1:
                    pass
                else:
                    new_p_fit = sim_dialog.plot_parameters
                    new_T_vals = sim_dialog.min_and_max_temps
                    
                    plot_to_make = ''.join(new_plot_type)
                    new_item_text = '{}, ({},{}), tQT: {}, Cr: {}, n: {}, t0: {}, Ueff: {}'.format(
                                    new_plot_type, new_T_vals[0], new_T_vals[1], new_p_fit['tQT'], new_p_fit['Cr'],
                                    new_p_fit['n'], new_p_fit['t0'], new_p_fit['Ueff'])
                    
                    self.ana_plot.ax.lines.remove(old_line)
                    
                    new_line = addPartialModel(self.ana_plot.fig,
                                               new_T_vals[0],
                                               new_T_vals[1],
                                               self.prepare_sim_dict_for_plotting(new_p_fit),
                                               plotType=plot_to_make)
                    
                    list_item_data = {'plot_type': new_plot_type,
                                      'p_fit': new_p_fit,
                                      'T_vals': new_T_vals,
                                      'line': new_line}
                    
                    self.ana_plot.canvas.draw()
                    
                    sim_item.setData(32, list_item_data)
                    sim_item.setText(new_item_text)
            else:
                pass
        
    def delete_sim(self):
        
        try:
            sim_item = self.list_of_simulations.selectedItems()[0]
        except IndexError:
            pass
        else:
            line_pointer = sim_item.data(32)['line']
            self.ana_plot.ax.lines.remove(line_pointer)
            self.ana_plot.canvas.draw()
            
            item_row = self.list_of_simulations.row(sim_item)
            sim_item = self.list_of_simulations.takeItem(item_row)
            
            del sim_item
        
    def plot_t_tau_on_axes(self):
        
        if self.data_used_pointer is not None:
            self.ana_plot.ax.lines.remove(self.data_used_pointer)
        if self.data_not_used_pointer is not None:
            self.ana_plot.ax.lines.remove(self.data_not_used_pointer)
        
        self.data_used_pointer, = self.ana_plot.ax.plot(1/self.used_T, np.log(self.used_tau), 'bo')
        self.data_not_used_pointer, = self.ana_plot.ax.plot(1/self.not_used_T, np.log(self.not_used_tau), 'ro')
        
        self.ana_plot.canvas.draw()
    
    def load_t_tau_data(self):
        
        if self.startUp:
            try:
                filename = sys.argv[1]
            except IndexError:
                pass
            finally:
                self.startUp = False
                return 0
        else:
            starting_directory = os.getcwd()
            filename_info = QFileDialog().getOpenFileName(self, 'Open file', starting_directory)
            filename = filename_info[0]
        
        if filename == '':
            pass
        else:
            self.data_file_origin = filename
        
        try:
            D = np.loadtxt(self.data_file_origin, skiprows=1)
        except (ValueError, OSError) as error:
            error_type = error.__class__.__name__
            if error_type == 'ValueError':
                msg = QMessageBox()
                msg.setWindowTitle('ValueError')
                msg.setIcon(QMessageBox.Warning)
                msg.setText('File format is not as expected')
                msg.exec_()
            elif error_type == 'OSError':
                pass
        else:
            self.set_new_t_tau(D[:,0], D[:,1])
            self.read_indices_for_used_temps()
            self.plot_t_tau_on_axes()
    
    def set_new_t_tau(self, T, tau):
        
        self.data_T = T
        self.data_tau = tau
    
    def prepare_sim_dict_for_plotting(self, p_fit_gui_struct):

        params = []
        quantities = []
        sigmas = [0]*5

        for key, val in p_fit_gui_struct.items():
            params.append(val)
            quantities.append(key)
        
        Ueff = params[quantities.index('Ueff')]
        params[quantities.index('Ueff')] = Ueff*scicon.Boltzmann
        
        p_fit_script_type = {'params': params,
                             'quantities': quantities,
                             'sigmas': sigmas}
        
        return p_fit_script_type
    
    def read_fit_type_cbs(self):
    
        list_of_checked = []
        if self.qt_cb.isChecked(): list_of_checked.append('QT')
        if self.raman_cb.isChecked(): list_of_checked.append('R')
        if self.orbach_cb.isChecked(): list_of_checked.append('O')
        fitToMake = ''.join(list_of_checked)
        
        return fitToMake
    
    def read_indices_for_used_temps(self):
        
        min_t = self.temp_line[1].value()
        max_t = self.temp_line[3].value()
        
        try:
            self.used_indices = [list(self.data_T).index(t) for t in self.data_T if t>=min_t and t<=max_t]
            
            self.used_T = self.data_T[self.used_indices]
            self.used_tau = self.data_tau[self.used_indices]
            
            self.not_used_T = np.delete(self.data_T, self.used_indices)
            self.not_used_tau = np.delete(self.data_tau, self.used_indices)
            
        except (AttributeError, TypeError):
            print('No data have been selected yet!')
    
    def fit_relaxation(self):
        """Reimplemented from fitRelaxation"""
        
        guess = getParameterGuesses(self.used_T, self.used_tau)
        perform_this_fit = self.read_fit_type_cbs()
        
        f = getFittingFunction(fitType=perform_this_fit)
        p0 = getStartParams(guess, fitType=perform_this_fit)
        
        popt, pcov = curve_fit(f, self.used_T, np.log(self.used_tau), p0)
        
        p_fit = readPopt(popt, pcov, fitType=perform_this_fit)
        
        return p_fit
        
    def set_new_temp_ranges(self):
    
        new_max_for_low = self.temp_line[3].value()
        new_min_for_high = self.temp_line[1].value()
        self.temp_line[1].setRange(0,new_max_for_low)
        self.temp_line[3].setRange(new_min_for_high,1000)
        
        self.read_indices_for_used_temps()
        if self.data_T is not None:
            self.plot_t_tau_on_axes()
        
    def make_the_fit(self):
        
        try:
            Tmin = self.temp_line[1].value()
            Tmax = self.temp_line[3].value()
            perform_this_fit = self.read_fit_type_cbs()
            assert Tmin != Tmax
            assert perform_this_fit != ''
            
            #fig3, p_fit = fitRelaxation(self.data_file_origin, (Tmin, Tmax), fitType=perform_this_fit)
            p_fit = self.fit_relaxation()
            self.fitted_parameters = p_fit
        
        except (AssertionError, RuntimeError, ValueError) as error:
            
            error_type = error.__class__.__name__
            if error_type == 'AssertionError':
                msg_text = 'Check your temperature and fit settings'
                msg_details = """Possible errors:
 - min and max temperatures are the same
 - no fit options have been selected"""
            elif error_type == 'RuntimeError':
                msg_text = 'This fit cannot be made within the set temperatures'
                msg_details = ''
            elif error_type == 'ValueError':
                msg_text = 'No file has been loaded'
                msg_details = ''
            
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Warning)
            msg.setWindowTitle('Fit aborted')
            msg.setText(msg_text)
            msg.setDetailedText(msg_details)
            msg.exec_()

class FitResultPlotStatus(QDialog):

    def __init__(self, list_input=None):
    
        super(FitResultPlotStatus, self).__init__()
        
        self.layout = QVBoxLayout()
        
        self.lbl_lo = QHBoxLayout()
        self.raw_lbl = QLabel('Raw')
        self.fit_lbl = QLabel('Fit')
        self.lbl_lo.addWidget(self.raw_lbl)
        self.lbl_lo.addWidget(self.fit_lbl)
        self.layout.addLayout(self.lbl_lo)
        
        self.scroll = QScrollArea(self)
        self.scroll.setWidgetResizable(True)
        self.layout.addWidget(self.scroll)
        
        self.content = QWidget(self.scroll)
        self.cont_lo = QVBoxLayout(self.content)
        self.content.setLayout(self.cont_lo)
        self.scroll.setWidget(self.content)
        
        self.checked_items = []
        
        num_of_temps = list_input.count()
        for idx in range(num_of_temps):
            item = list_input.item(idx)
            item_lo = QHBoxLayout()
            item_data = item.data(32)
            
            item_fit_bool = item_data['fit']
            item_raw_bool = item_data['raw']
            item_txt = item_data['temp']
            
            raw_checked = QCheckBox()
            fit_checked = QCheckBox()
            temp = QLabel('{:5.2f}K'.format(item_data['temp']))
            
            item_lo.addWidget(temp)
            item_lo.addWidget(raw_checked)
            item_lo.addWidget(fit_checked)
            
            self.checked_items.append([raw_checked, fit_checked])
            
            raw_checked.setChecked(item_raw_bool)
            fit_checked.setChecked(item_fit_bool)
            
            self.cont_lo.addLayout(item_lo)
        
        self.state_btn_lo = QHBoxLayout()
        
        self.check_all_btn = QPushButton('Check all')
        self.check_all_btn.clicked.connect(self.check_all_function)
        
        self.uncheck_all_btn = QPushButton('Uncheck all')
        self.uncheck_all_btn.clicked.connect(self.uncheck_all_function)
        
        self.state_btn_lo.addWidget(self.uncheck_all_btn)
        self.state_btn_lo.addWidget(self.check_all_btn)
        
        self.layout.addLayout(self.state_btn_lo)
        
        self.judge_btn_lo = QHBoxLayout()
        
        self.states_reject_btn = QPushButton('Cancel')
        self.states_reject_btn.clicked.connect(self.reject)
        self.judge_btn_lo.addWidget(self.states_reject_btn)
        
        self.states_accept_btn = QPushButton('Ok')
        self.states_accept_btn.clicked.connect(self.accept)
        self.judge_btn_lo.addWidget(self.states_accept_btn)
        
        self.layout.addLayout(self.judge_btn_lo)
        
        self.setLayout(self.layout)
        self.show()
        
    def check_all_function(self):
    
        for sublist in self.checked_items:
            sublist[0].setChecked(True)
            sublist[1].setChecked(True)
        
    def uncheck_all_function(self):
    
        for sublist in self.checked_items:
            sublist[0].setChecked(False)
            sublist[1].setChecked(False)
        
class FitResultPlotStatus2(QDialog):
    # Old class that implemented a dialog to change one list item at a time
    
    def __init__(self, plotting_dict=None):
    
        super(FitResultPlotStatus, self).__init__()
        
        self.setWindowTitle('Update plot status')
        self.layout = QGridLayout()
        
        self.raw_lbl = QLabel('Raw')
        
        self.raw_cb = QCheckBox()
        self.raw_cb.setChecked(plotting_dict['raw'])
        
        self.fit_lbl = QLabel('Fitted')
        
        self.fit_cb = QCheckBox()
        self.fit_cb.setChecked(plotting_dict['fit'])
        
        self.cancel_btn = QPushButton('Cancel')
        self.cancel_btn.clicked.connect(self.reject)
        
        self.ok_btn = QPushButton('Ok')
        self.ok_btn.clicked.connect(self.accept)
        
        self.layout.addWidget(self.raw_lbl,0,0)
        self.layout.addWidget(self.raw_cb,1,0)
        self.layout.addWidget(self.cancel_btn,2,0)
        self.layout.addWidget(self.fit_lbl,0,1)
        self.layout.addWidget(self.fit_cb,1,1)
        self.layout.addWidget(self.ok_btn,2,1)
        
        self.setLayout(self.layout)
        
        self.show()
        
class ParamDialog(QDialog):

    def __init__(self,
                 parent=None,
                 param_dict=None):
                 
        super(ParamDialog, self).__init__()
        
        self.setWindowTitle('Fitted parameters')
        self.dialog_layout = QVBoxLayout()
        self.param_labels = {}
        
        for val in param_dict['quantities']:
            multiplier = 1
            idx = param_dict['quantities'].index(val)
            
            current_label = QLabel()
            if val == 'Ueff': multiplier = scicon.Boltzmann
            current_label.setText('{}: {:6.3e} +/- {:6.3e}'.format(val,
                                                                   param_dict['params'][idx]/multiplier,
                                                                   param_dict['sigmas'][idx]/multiplier))
                                                                   
            self.dialog_layout.addWidget(current_label)
        
        self.setLayout(self.dialog_layout)
        self.show()

class SimulationDialog(QDialog):

    def __init__(self,
                 parent=None,
                 fitted_parameters=None,
                 plot_type_list=[],
                 plot_parameters={'tQT': 0.01, 'Cr': 0.00, 'n': 0.00, 't0': 0.00, 'Ueff': 0.00},
                 min_and_max_temps=[0]*2):
    
        super(SimulationDialog, self).__init__()
        
        self.setWindowTitle('Add simulation')
        
        # Containers for objects
        self.lineedit_inputs = {}
        
        self.headline_font = QFont()
        self.headline_font.setBold(True)
        
        self.plot_type_list = plot_type_list
        self.plot_parameters = plot_parameters
        
        self.min_and_max_temps = min_and_max_temps
        
        self.layout = QVBoxLayout()
        
        if fitted_parameters is not None:
            self.use_fit_cb = QCheckBox('Use fitted parameters')
            self.use_fit_cb.stateChanged.connect(self.fit_take_control)
            self.layout.addWidget(self.use_fit_cb)
            
            self.fitted_parameters = {}
            for quant in fitted_parameters['quantities']:
                try:
                    idx = fitted_parameters['quantities'].index(quant)
                    param = fitted_parameters['params'][idx]
                except IndexError:
                    param = None
                finally:
                    if quant=='Ueff':
                        param/=scicon.Boltzmann
                    self.fitted_parameters[quant] = param
            
        # Controls to play with temperature
        self.temp_headline = QLabel('Temperature')
        self.temp_headline.setFont(self.headline_font)
        self.layout.addWidget(self.temp_headline)
        
        self.temp_hbl = QHBoxLayout()
        
        self.temp_min = QDoubleSpinBox()
        self.temp_min.setValue(min_and_max_temps[0])
        self.temp_min.editingFinished.connect(self.temp_interval_changed)
        self.temp_hbl.addWidget(self.temp_min)
        
        self.temp_max = QDoubleSpinBox()
        self.temp_max.setValue(min_and_max_temps[1])
        self.temp_max.editingFinished.connect(self.temp_interval_changed)
        self.temp_hbl.addWidget(self.temp_max)
        
        self.layout.addLayout(self.temp_hbl)
        
        # Controls for which type of plot to consider
        self.plot_headline = QLabel('Plot type to make')
        self.plot_headline.setFont(self.headline_font)
        self.layout.addWidget(self.plot_headline)
        
        self.plot_type_hbl = QHBoxLayout()
        
        self.use_orbach = QCheckBox('Orbach')
        if 'O' in self.plot_type_list: self.use_orbach.setChecked(True)
        self.use_orbach.clicked.connect(self.plot_type_changed)
        self.plot_type_hbl.addWidget(self.use_orbach)
        
        self.use_raman = QCheckBox('Raman')
        if 'R' in self.plot_type_list: self.use_raman.setChecked(True)
        self.use_raman.clicked.connect(self.plot_type_changed)
        self.plot_type_hbl.addWidget(self.use_raman)
        
        self.use_qt = QCheckBox('QT')
        if 'QT' in self.plot_type_list: self.use_qt.setChecked(True)
        self.use_qt.clicked.connect(self.plot_type_changed)
        self.plot_type_hbl.addWidget(self.use_qt)
        
        self.layout.addLayout(self.plot_type_hbl)
        
        self.sim_vals_layout = QFormLayout()
        
        self.tqt_val = QLineEdit()
        self.lineedit_inputs['tQT'] = self.tqt_val
        self.tqt_val.setValidator(QDoubleValidator())
        self.tqt_val.setText(str(self.plot_parameters['tQT']))
        self.tqt_val.editingFinished.connect(self.param_values_changed)
        self.sim_vals_layout.addRow('t_QT', self.tqt_val)
        
        self.cr_val = QLineEdit()
        self.lineedit_inputs['Cr'] = self.cr_val
        self.cr_val.setValidator(QDoubleValidator())
        self.cr_val.setText(str(self.plot_parameters['Cr']))
        self.cr_val.editingFinished.connect(self.param_values_changed)
        self.sim_vals_layout.addRow('C_R', self.cr_val)
        
        self.n_val = QLineEdit()
        self.lineedit_inputs['n'] = self.n_val
        self.n_val.setValidator(QDoubleValidator())
        self.n_val.setText(str(self.plot_parameters['n']))
        self.n_val.editingFinished.connect(self.param_values_changed)
        self.sim_vals_layout.addRow('n', self.n_val)
        
        self.t0_val = QLineEdit()
        self.lineedit_inputs['t0'] = self.t0_val
        self.t0_val.setValidator(QDoubleValidator())
        self.t0_val.setText(str(self.plot_parameters['t0']))
        self.t0_val.editingFinished.connect(self.param_values_changed)
        self.sim_vals_layout.addRow('t0', self.t0_val)
        
        self.Ueff_val = QLineEdit()
        self.lineedit_inputs['Ueff'] = self.Ueff_val
        self.Ueff_val.setValidator(QDoubleValidator())
        self.Ueff_val.setText(str(self.plot_parameters['Ueff']))
        self.Ueff_val.editingFinished.connect(self.param_values_changed)
        self.sim_vals_layout.addRow('U_eff', self.Ueff_val)
        
        self.layout.addLayout(self.sim_vals_layout)
        
        # Making control buttons at the end
        self.button_layout = QHBoxLayout()
        
        self.cancel_btn = QPushButton('Cancel')
        self.cancel_btn.setAutoDefault(False)
        self.cancel_btn.clicked.connect(self.reject)
        self.button_layout.addWidget(self.cancel_btn)
        
        self.accept_btn = QPushButton('Ok')
        self.accept_btn.clicked.connect(self.accept)
        self.button_layout.addWidget(self.accept_btn)
        
        self.layout.addLayout(self.button_layout)
        
        self.setLayout(self.layout)
        
        self.show()
    
    def fit_take_control(self):
        
        if self.use_fit_cb.isChecked():
            
            for key, val in self.fitted_parameters.items():
                if val is not None:
                    self.lineedit_inputs[key].setText('{}'.format(val))
                self.lineedit_inputs[key].setReadOnly(True)
            
        else:
            self.tqt_val.setReadOnly(False)
            self.cr_val.setReadOnly(False)
            self.n_val.setReadOnly(False)
            self.t0_val.setReadOnly(False)
            self.Ueff_val.setReadOnly(False)
        
        self.param_values_changed()
    
    def param_values_changed(self):
        
        self.plot_parameters['tQT'] = float(self.tqt_val.text())
        self.plot_parameters['Cr'] = float(self.cr_val.text())
        self.plot_parameters['n'] = float(self.n_val.text())
        self.plot_parameters['t0'] = float(self.t0_val.text())
        self.plot_parameters['Ueff'] = float(self.Ueff_val.text())
        
    def plot_type_changed(self):
        
        self.plot_type_list = []
        if self.use_qt.isChecked(): self.plot_type_list.append('QT')
        if self.use_raman.isChecked(): self.plot_type_list.append('R')
        if self.use_orbach.isChecked(): self.plot_type_list.append('O')
        
    def temp_interval_changed(self):
        
        try:
            self.min_and_max_temps[0] = self.temp_min.value()
            self.min_and_max_temps[1] = self.temp_max.value()
            assert self.min_and_max_temps[0]<=self.min_and_max_temps[1]
        except AssertionError:
            pass

class PlottingWindow(QWidget):

    def __init__(self):
    
        super(PlottingWindow, self).__init__()
        
        self.layout = QVBoxLayout()
        
        self.fig = Figure()
        self.canvas = FigureCanvas(self.fig)
        self.tools = NavigationToolbar(self.canvas, self)
        self.ax = self.fig.add_subplot(111)
        self.fig.subplots_adjust(left=0.05, bottom=0.05, right=0.95, top=0.95)
        
        self.layout.addWidget(self.canvas)
        
        self.tool_lo = QHBoxLayout()
        self.tool_lo.addWidget(self.tools)
        self.tool_lo.addStretch()
        
        self.reset_axes_btn = QPushButton('Reset axes')
        self.reset_axes_btn.clicked.connect(self.reset_axes)
        self.tool_lo.addWidget(self.reset_axes_btn)
        self.layout.addLayout(self.tool_lo)
        
        self.setLayout(self.layout)
        
    def reset_axes(self):
       
       s = 0
       if len(self.ax.lines)<1: pass
       else:
           while True:
               start = self.ax.lines[s]
               if len(start.get_xdata())<1:
                   s += 1
               else:
                   break
           
           x = start.get_xdata()
           y = start.get_ydata()
           
           new_x = [x.min(), x.max()]
           new_y = [y.min(), y.max()]
           
           for i in range(s, len(self.ax.lines)):
               x = self.ax.lines[i].get_xdata()
               y = self.ax.lines[i].get_ydata()
               
               if len(x)>1 and len(y)>1:
                   if x.min()<new_x[0]: new_x[0] = x.min()
                   if x.max()>new_x[1]: new_x[1] = x.max()
                   if y.min()<new_y[0]: new_y[0] = y.min()
                   if y.max()>new_y[1]: new_y[1] = y.max()
           
           if new_x[0] == new_x[1]:
               new_x[0] -= 0.5
               new_x[1] += 0.5
           if new_y[0] == new_y[1]:
               new_y[0] -= 0.5
               new_y[1] += 0.5
               
           self.ax.set_xlim(new_x[0]-0.05*(new_x[1]-new_x[0]),new_x[1]+0.05*(new_x[1]-new_x[0]))
           self.ax.set_ylim(new_y[0]-0.05*(new_y[1]-new_y[0]),new_y[1]+0.05*(new_y[1]-new_y[0]))
           
           self.canvas.draw()

class AboutDialog(QDialog):
    
    def __init__(self, info):
    
        super(AboutDialog, self).__init__()
        
        self.layout = QVBoxLayout()
        
        self.setWindowTitle('About')
        
        self.author_lbl = QLabel('Written by {}'.format(info['author']))
        self.layout.addWidget(self.author_lbl)
        
        self.web_lbl = QLabel('<a href={}>Molecular magnetism at AU</a>'.format(info['webpage']))
        self.web_lbl.setOpenExternalLinks(True)
        self.layout.addWidget(self.web_lbl)
        
        self.pers_lbl = QLabel('Personal <a href={}>webpage</a>'.format(info['personal']))
        self.pers_lbl.setOpenExternalLinks(True)
        self.layout.addWidget(self.pers_lbl)
        
        self.setLayout(self.layout)
        self.show()
          
if __name__ == '__main__':
    
    myappid = 'AC Processing v1.0'
    ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)
    
    app = QApplication(sys.argv)
    w = ACGui()
    sys.exit(app.exec_())
        
        
        